#
# graveyard.R, 26 Nov 18
# Data from:
# https://killedbygoogle.com
# Cody Ogden
#
# Example from:
# Evidence-based Software Engineering: based on the publicly available data
# Derek M. Jones
#
# TAG

source("ESEUR_config.r")


library(jsonlite)


pal_col=rainbow(2)


grave=fromJSON(paste0(ESEUR_dir, "ecosystems/graveyard.json"))

grave$dateOpen=as.Date(grave$dateOpen, format="%Y-%m-%d")
grave$dateClose=as.Date(grave$dateClose, format="%Y-%m-%d")

grave$lifetime=as.numeric(grave$dateClose-grave$dateOpen+1)

soft_only=subset(grave, !(grepl("Pixel", grave$description) |
			  grepl("Nexus", grave$description) |
			  grepl("Google Glass", grave$description)))

lifetime=data.frame(available=nrow(soft_only):1, age=sort(soft_only$lifetime))

plot(sort(soft_only$lifetime), nrow(soft_only):1, log="y", col=pal_col[1],
	xaxs="i",
	xlab="Days", ylab="Software systems\n")

# Additive error gives a fit with smaller percentage of residual deviance
# But is a multiplicative error more realistic?
# l_mod=glm(log(available) ~ age, data=lifetime)
l_mod=glm(available ~ age, data=lifetime, family=poisson)
# summary(l_mod)
pred=predict(l_mod)
lines(lifetime$age, exp(pred), col=pal_col[1])

# hist(round(grave$lifetime/365.25), breaks=20)
# 
# plot(density(grave$lifetime, cut=0, n=32))


mainf=read.csv(paste0(ESEUR_dir, "economics/system-lifetime.csv.xz"), as.is=TRUE)

mainf$days=mainf$years*365.25

num_years=nrow(mainf)
t=cumsum(mainf$projects)
num_proj=max(t)
mainf$remaining=num_proj-t
points(mainf$days, mainf$remaining, col=pal_col[2])

mainf=head(mainf, n=-1)

# IBM_mod=glm(log(remaining) ~ days, data=mainf)
# summary(IBM_mod)
# lines(mainf$days, exp(predict(IBM_mod)), col=pal_col[2])

# Additive error gives a fit with smaller percentage of residual deviance
yr1_mod=glm(remaining ~ days, data=mainf, family=poisson)
summary(yr1_mod)
lines(mainf$days, predict(yr1_mod, type="response"), col=pal_col[2])

# yr2_mod=glm(remaining ~ days+I(days^2), data=mainf, family=poisson)
# summary(yr2_mod)
# lines(mainf$days, predict(yr2_mod, type="response"), col=pal_col[2])

# a power law model just to see what it looks like
# q=nls(remaining ~ a*b^years, data=mainf, start=list(a=100, b=0.8))
# summary(q)

# lines(predict(q), col=pal_col[3])

legend(x="bottomleft", legend=c("Google", "Mainframe"), bty="n", fill=pal_col, cex=1.2)


